function autoorder_demo(varargin)
%AUTOORDER_DEMO  Automatic model order selection demos
%       Demo function for autoorder
% 
%       User interface to start a bunch of demos which illustrate the 
%       automatic model selection method.
%
%       varargin{1}: 'init','pushbutton',...
%       varargin{2}: 'sme' if sme called

%       Written by Gyula Simon, 1998
%       Copyright (c) I. Kollar and Vrije Universiteit Brussel, ELEC, 1991-2002
%       All rights reserved.
%       $Revision: $
%       Last modified: 12-Aug-2002

persistent rel_speed %benchmark speed of this computer
if isempty(rel_speed)
  rel_speed=fdident('private','fdbench');
  rel_speed=fdident('private','fdbench');
end
persistent truetimes %true execution times of demos
persistent timestrall %predicted execution times of demos
persistent demonos %serial numbers of demonstrations
if isempty(truetimes), truetimes={}; end
if isempty(timestrall), timestrall={}; end
if isempty(demonos), demonos={}; end
%while length(truetimes)<length(timestrall), timestrall(end)=[]; end
%while length(truetimes)<length(demonos), demonos(end)=[]; end
if (nargin==1)&isstr(varargin{1})&strcmp(varargin{1},'preload'), return, end

hListstr0={...
    'Demos for Automatic Model Selection',...
    '', ...
    'Here you find some measurement data and some simulated data', ...
    'to illustrate the automatic model selection algorithm.', ...
    '', ...   
    'Select one of the data by pressing a pushbutton above to read ', ...
    'more on the specific experiment.', ...
    '', ...   
    'The run-time data are estimated for this machine. Originally, these were',...
    'measured on a Pentium III Win2000 1 GHz Computer (fdident(''fdbench'') ~ 21.8).', ...
    '', ...
    'Next, press ''Start'' to see how order selection works...', ...
  };
pf=[1 110 598 285];
if nargin==0 | isequal(varargin{1}, 'init')
  hFig=findobj(allchild(0), 'flat', 'Tag','AutoModelSelectionDemoFig');
  if ishandle(hFig), close(hFig), end
  hFig = figure('Units','points', ...
    'Name','Demos for Automatic Model Selection', ...
    'NumberTitle','off', ...
    'Units','points', ...
    'Position',pf, ...
    'Tag','AutoModelSelectionDemoFig', ...
    'HandleVisibility','off', ...
    'Resize','off');
  
  Ver=version;
  if str2num(Ver(1:3))>=5.3
    set(hFig,'ToolBar','none');
  end
  
  pL=[24 22 283 130]; pL=pL+[0,0,0,54];
  pL=[24 22 400 200]; 
  hList = uicontrol('Parent',hFig, ...
    'Units','points', ...
    'Position',pL, ...
    'String',' ', ...
    'Style','listbox', ...
    'Tag','ListBox', ...
    'Value',1);

  if 1
    demos={'Robotarm', ...
        'Electrical machine', ...
        'Mechanical system', ...
        'BK filter - Full band', ...
        'BK filter - Selected 1', ...
        'BK filter - Selected 2', ...
        'Wilkinson 20/20', ...
        'Wilkinson 50/50', ...
        'Wilkinson 100/100', ...
        'Bk filter - New meas. I.', ...
        'Bk filter - New meas. II.', ...
        'Bk filter - New meas. III.', ...
        'Nonlinear I.', ...
        'Nonlinear II.', ...
        'CD player'};
    

    ppu=[24,0.8*pf(4),100,27];
    hpopup = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String',[{'Select a demo...'},demos], ...
      'Position',ppu, ...
      'Callback', [mfilename,'(''popupmenu'',get(gcbo,''value''))'], ...
      'Tag','popupmenu',...
      'style','popupmenu');
    
    %pf=[1 110 598 285];
    ptxt=ppu+[0,25,20,-10];
    htext=uicontrol('parent',hFig,...
      'units','points',...
      'style','text',...
      'position',ptxt,...
      'tag','demo_select_txt',...
      'horizontalalignment','left',...
      'backgroundcolor',get(hFig,'color'),...
      'string','Selected demonstration:'...
      );
  
  else
    p13=[24 265 100 27]; p13(2)=pf(4)-27-23;
    hPush1 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','Robotarm', ...
      'Position',p13, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton1');
    p12=p13+[0,-43,0,0];
    hPush2 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','Electrical machine', ...
      'Position',p12, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton2');
    p11=p13+[0,-2*43,0,0];
    hPush3 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','Mechanical system', ...
      'Position',p11, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton3');
    p23=p13+[114,0,0,0];
    hPush4 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','BK filter - Full band', ...
      'Position',p23, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton4');
    p22=p23+[0,-43,0,0];
    hPush5 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','BK filter - Selected 1', ...
      'Position',p22, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton5');
    p21=p23+[0,-2*43,0,0];
    hPush6 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','BK filter - Selected 2', ...
      'Position',p21, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton6');
    p33=p13+[2*114,0,0,0];
    hPush7 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','Wilkinson 20/20', ...
      'Position',p33, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton7');
    p32=p33+[0,-43,0,0];
    hPush8 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','Wilkinson 50/50', ...
      'Position',p32, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton8');
    p31=p33+[0,-2*43,0,0];
    hPush9 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','Wilkinson 100/100', ...
      'Position',p31, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton9');
    p43=p13+[3*114,0,0,0];
    hPush10 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','Bk filter - New meas. I.', ...
      'Position',p43, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton10');
    p42=p43+[0,-43,0,0];
    hPush11 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','Bk filter - New meas. II.', ...
      'Position',p42, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton11');
    p41=p43+[0,-2*43,0,0];
    hPush12 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','Bk filter - New meas. III.', ...
      'Position',p41, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton12');
    p53=p13+[4*114,0,0,0];
    hPush13 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','Nonlinear I.', ...
      'Position',p53, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton13');
    p52=p53+[0,-43,0,0];
    hPush14 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','Nonlinear II.', ...
      'Position',p52, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton14');
    p51=p53+[0,-2*43,0,0];
    hPush15 = uicontrol('Parent',hFig, ...
      'Units','points', ...
      'ListboxTop',0, ...
      'String','CD player', ...
      'Position',p51, ...
      'Callback', [mfilename '( get(gcbo, ''tag'') )'], ...
      'Tag','Pushbutton15');
  end
  
  pbleft=433;
  hPushClose = uicontrol('Parent',hFig, ...
    'Units','points', ...
    'ListboxTop',0, ...
    'Position',[pbleft 20 71 27], ...
    'String','Close',...
    'Callback', [mfilename '(''close'')'], ...
    'Tag','PushbuttonClose');
  hPushHelp = uicontrol('Parent',hFig, ...
    'Units','points', ...
    'ListboxTop',0, ...
    'Position',[pbleft 55 71 27], ...
    'String','Help',...
    'Callback', [mfilename '(''help'')'], ...
    'Tag','PushbuttonHelp');
  hPushStart = uicontrol('Parent',hFig, ...
    'Units','points', ...
    'ListboxTop',0, ...
    'Position',[pbleft 152-27 71 27], ...
    'String','Start demo',...
    'Interrupt','off',...
    'Enable', 'off', ...
    'Callback', [mfilename '(''start'')'], ...
    'Tag','PushbuttonStart');

   hPushAbort = uicontrol('Parent',hFig, ...
    'Units','points', ...
    'ListboxTop',0, ...
    'Position',[pbleft 152-27-30 71 27], ...
    'String','Abort',...
    'Interrupt','off',...
    'Enable', 'off', ...
    'Callback', [mfilename '(''abort'')'], ...
    'Tag','PushbuttonAbort');

  set(hList, 'string', hListstr0);
  drawnow
  
  if nargin>=2, caller=varargin{2}; else caller=''; end
  if strncmp(version,'5',1)
    setuprop(hFig,'CALLER_FCN',caller)
  else
    setappdata(hFig,'CALLER_FCN',caller)
  end
    
else
  hFig=findobj(allchild(0), 'flat', 'Tag','AutoModelSelectionDemoFig');
  if any(findstr(varargin{1},'Pushbutton'))|any(findstr(varargin{1},'popupmenu'))
    load autoorder_data.mat
    if any(findstr(varargin{1},'Pushbutton'))
      ix=str2num(varargin{1}(11:end));
    else
      ix=varargin{2}-1;
    end
    if ix==0, set(findall(0,'Tag','PushbuttonStart'),'enable','off')
    else set(findall(0,'Tag','PushbuttonStart'),'enable','on')
    end
    switch ix
    case 0 %init
      note=hListstr0;
    case 1                      % Robotarm
      timeref=17.6; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      else timestr=sprintf('%.2g min',timeown/60);
      end
      fdata=robotarm_freqdata_agv;
      note={...
          ['Demo ',num2str(ix),': Robotarm'], ...
          '  ', ...
          '  Source: 10 sync experiments, 100 freqlines.', ...
          '  ', ...
          '  Slight nonlinear distortion.', ...
          '  ', ...
          '  Expected order: 4/6 or 6/4.', ...
          '  ', ...
          ['  Estimation speed: fast (about ',timestr,')'], ...
        };
    case 2                      % Electrical machine
      timeref=17.4; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      else timestr=sprintf('%.2g min',timeown/60);
      end
      fdata=emach_agv;
      note={...
          ['Demo ',num2str(ix),': Electrical machine'], ...
          '  ', ...
          '  Source: 10 sync experiments, 78 freqlines', ...
          '  ', ...
          '  Nonlinear distortion.', ...
          '  ', ...
          '  Expected order: around 4/4.', ...
          '  ', ...
          ['  Estimation speed: fast (about ',timestr,')'], ...
        };
    case 3                      % Mechanical system
      timeref=252.3; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      else timestr=sprintf('%.2g min',timeown/60);
      end
      fdata=mech_agv;
      note={...
          ['Demo ',num2str(ix),': Mechanical system'], ...
          '  ', ...
          '  Source: 35 sync experiments, 107 freqlines.', ...
          '  ', ...
          '  Strong nonlinear distortion.', ...
          '  ', ...
          '  Expected order: around 16/16.', ...
          '  ', ...
          ['  Estimation speed: medium (about ',timestr,')'], ...
        };
    case 4                      % Bruel & Kjaer filter - full band
      timeref=293.3; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      else timestr=sprintf('%.2g min',timeown/60);
      end
      fdata=bkdata_all_agv;
      note={...
          ['Demo ',num2str(ix),': Bruel & Kjaer filter - full measured band'], ...
          '  ', ...
          '  Source: 8 sync experiments, 512 freqlines.', ...
          '  ', ...
          '  Measurement errors close to DC and at high freqs.', ...
          '  ', ...
          '  Nonlinear distortion above noise level.', ...
          '  ', ...
          '  Expected order: 5/6+????.', ...
          '  ', ...
          ['  Estimation speed: medium (about ',timestr,')'], ...
        };
    case 5                      % Bruel & Kjaer filter - selected band
      timeref=121.7; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      else timestr=sprintf('%.2g min',timeown/60);
      end
      fdata=bkdata_sel_agv;
      note={...
          ['Demo ',num2str(ix),': Bruel & Kjaer filter - selected band'], ...
          '  ', ...
          '  Source: 8 sync experiments, 311 freqlines.', ...
          '  ', ...
          '  Suspicious frequency lines deleted (at low and high freq).', ...
          '  ', ...
          '  Nonlinear distortion above noise level.', ...
          '  ', ...
          '  Expected order: 5/6.', ...
          '  ', ...
          ['  Estimation speed: medium (about ',timestr,')'], ...
        };
    case 6                      % Bruel & Kjaer filter - strictly selected band
      timeref=32.6; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      else timestr=sprintf('%.2g min',timeown/60);
      end
      fdata=bkdata_selsel_agv;
      note={...
          ['Demo ',num2str(ix),': Bruel & Kjaer filter - strictly selected band'], ...
          '  ', ...
          '  Source: 8 sync experiments, 311 freqlines.', ...
          '  ', ...
          '  Lines deleted with tf < -20 dB.', ...
          '  ', ...
          '  Nonlinear distortion above noise level.', ...
          '  ', ...
          '  Expected order: 5/6.', ...
          '  ', ...
          ['  Estimation speed: fast (about ',timestr,')'], ...
        };
    case 7                      % Simulated Wilkinson 20/20 system
      timeref=50.4; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      else timestr=sprintf('%.2g min',timeown/60);
      end
      fdata=wilk20;
      note={...
          ['Demo ',num2str(ix),': Simulated 20/20 Wilkinson system'], ...
          '  ', ...
          '  Source: 10 sync experiments, 201 freqlines.', ...
          '  ', ...
          '  Expected order: 20/20.', ...
          '  ', ...
          ['  Estimation speed: fast (about ',timestr,')'], ...
        };
    case 8                      % Simulated Wilkinson 50/50 system
      timeref=273.8; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      elseif timeown<3600, timestr=sprintf('%.2g min',timeown);
      else timestr=sprintf('%.2g hours',timeown/60/60);
      end
      fdata=wilk50;
      note={...
          ['Demo ',num2str(ix),': Simulated 50/50 Wilkinson system'], ...
          '  ', ...
          '  Source: 10 sync experiments, 501 freqlines.', ...
          '  ', ...
          '  Expected order: 50/50.', ...
          '  ', ...
          '  Note, that the direct estimation of the 50/50 system fails. ', ...
          '     The order is first increased to 65/65, then the peeling can ', ...
          '     find the exact 50/50 system.', ...
          '  ', ...
          ['  Estimation speed: medium (about ',timestr,')'], ...
        };
    case 9                      % Simulated Wilkinson 100/100 system
      timeref=2963; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      elseif timeown<3600, timestr=sprintf('%.2g min',timeown);
      else timestr=sprintf('%.2g hours',timeown/60/60);
      end
      fdata=wilk100;
      note={...
          ['Demo ',num2str(ix),': Simulated 100/100 Wilkinson system'], ...
          '  ', ...
          '  Source: 10 sync experiments, 1001 freqlines.', ...
          '  ', ...
          '  Expected order: 100/100.', ...
          '  ', ...
          '  Note, that the direct estimation of the 100/100 system fails. ', ...
          '     The order is first increased to 141/141 (it is the order which ',...
          '     was high enough to model the data correctly usig the direct '...
          '     approach), then the peeling can find the exact 100/100 system.', ...
          '  ', ...
          ['  Estimation speed: slow (about ',timestr,')'], ...
        };
    case 10                      % New measurement on th B&K filter I.
      timeref=40.6; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      elseif timeown<3600, timestr=sprintf('%.2g min',timeown);
      else timestr=sprintf('%.2g hours',timeown/60/60);
      end
      fdata=bknew1_agv;
      note={...
          ['Demo ',num2str(ix),': Bruel & Kjaer filter - new measurement (data BenK400mV179K /1)'], ...
          '  ', ...
          '  Source: 10 sync experiments, 185 freqlines.', ...
          '  ', ...
          '  Excitation: odd-odd random multisine', ...
          '  ', ...
          '  Nonlinear distortion above noise level.', ...
          '  ', ...
          '  Expected order: 6/6', ...
          '  ', ...
          ['  Estimation speed: fast (about ',timestr,')'], ...
        };
    case 11                      % New measurement on th B&K filter II.
      timeref=1520; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      elseif timeown<3600, timestr=sprintf('%.2g min',timeown);
      else timestr=sprintf('%.2g hours',timeown/60/60);
      end
      fdata=bknew2_agv;
      note={...
          ['Demo ',num2str(ix),': Bruel & Kjaer filter - new measurement (data BenK400mV179K /3)'], ...
          '  ', ...
          '  Source: 10 sync experiments, 737  freqlines.', ...
          '  ', ...
          '  Excitation: consecutive random multisine', ...
          '  ', ...
          '  Nonlinear distortion above noise level.', ...
          '  ', ...
          '  Note: The coarse step fails in this this demo. Instead of', ...
          '        decreasing the initial 50/50 order, it is first increased.', ...
          '        The peeling, however, finds the correct solution.', ...
          '  ', ...
          '  Expected order: 6/6', ...
          '  ', ...
          ['  Estimation speed: slow (about ',timestr,')'], ...
        };
    case 12                      % New measurement on th B&K filter III.
      timeref=32.9; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      else timestr=sprintf('%.2g min',timeown/60);
      end
      fdata=bknew3_agv;
      note={...
          ['Demo ',num2str(ix),': Bruel & Kjaer filter - new measurement (data BenK400mV179K /5)'], ...
          '  ', ...
          '  Source: 10 sync experiments, 185 freqlines.', ...
          '  ', ...
          '  Excitation: special odd random multisine', ...
          '  (designed to detect nonlinearities)', ...
          '  ', ...
          '  Nonlinear distortion above noise level.', ...
          '  ', ...
          '  Expected order: 6/6', ...
          '  ', ...
          ['  Estimation speed: fast (about ',timestr,')'], ...
        };
    case 13                      % nonlinear system I.
      timeref=416.2; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      else timestr=sprintf('%.2g min',timeown/60);
      end
      fdata=nldata1_agv;
      note={...
          ['Demo ',num2str(ix),': Nonlinear system simulated by an electric circuit (data SNLS80mV /2)'], ...
          '  ', ...
          '  Source: 10 sync experiments, 168 freqlines.', ...
          '  ', ...
          '  Excitation: odd multisine with minimum crest factor', ...
          '  ', ...
          '  Low SNR and high nonlinear distortion.', ...
          '  See the high difference between the theoretical and the observed Cf''s.', ...
          '  Compare with demo Nonlinear II.', ...
          '  ', ...
          '  Expected order: 2/0', ...
          '  ', ...
          ['  Estimation speed: medium (about ',timestr,')'], ...
        };
    case 14                      % nonlinear system II.
      timeref=130.9; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      else timestr=sprintf('%.2g min',timeown/60);
      end
      fdata=nldata2_agv;
      note={...
          ['Demo ',num2str(ix),': Nonlinear system simulated by an electric circuit (data SNLS80mV /3)'], ...
          '  ', ...
          '  Source: 10 sync experiments, 1342 freqlines.', ...
          '  ', ...
          '  Excitation: odd multisine with random phase', ...
          '  ', ...
          '  Low SNR and high nonlinear distortion.', ...
          '  Nonlinear effects taken into consideration in the variance,', ...
          '  so the theoretical and the observed Cf''s are close.', ...
          '  Compare with demo Nonlinear I.', ...
          '  ', ...
          '  Expected order: 2/0', ...
          '  ', ...
          ['  Estimation speed: medium (about ',timestr,')'], ...
        };
    case 15                      % CD player
      timeref=156.9; timeown=timeref*rel_speed/21.8;
      if timeown<60, timestr=sprintf('%.2g s',timeown);
      else timestr=sprintf('%.2g min',timeown/60);
      end
      fdata=cdplayer_agv;
      note={...
          ['Demo ',num2str(ix),': Radial servo system of a CD player (data CDTrack1_4096x64)'], ...
          '  ', ...
          '  Source: 16 experiments, 281 freqlines.', ...
          '  ', ...
          '  Expected order: around 20/20 ??', ...
          '  ', ...
          ['  Estimation speed: medium (about ',timestr,')'], ...
        };
    end
    timestrall=[timestrall;{timestr}];
    demonos=[demonos;{ix}];
    note=[note, {''}, {''}, {'Press ''Start demo'' to begin...'}];
    hList=findobj(allchild(hFig), 'flat', 'Tag','ListBox');
    set(hList, 'string', note, 'value', 1);
    hStart=findobj(allchild(hFig), 'flat', 'Tag','PushbuttonStart');
    %set(hStart, 'enable', 'on');
    if ix>0, set(hFig, 'userdata', fdata), end
    
  elseif isequal(varargin{1}, 'abort')
    helis=findall(0,'tag','elis_window')
    iterctrl('Abort',helis)
    
  elseif isequal(varargin{1}, 'start')
    hA=findobj(hFig,'Tag','PushbuttonAbort');
    set(hA,'enable','on')
    dem=popupstr(findobj(hFig,'tag','popupmenu'));
    disp(['Running automatic order selection demonstration ''',dem,''' ...'])
    fdata=get(hFig, 'userdata');
    fv=fdata.freqpoints; ind=find(fv<=0); if ~isempty(ind), fv(ind)=[]; end  
    runmod='';
    if max(fv)/min(fv)>5e3, runmod.plotfreq='log'; end
    ha=findall(0,'type','figure','tag','aided_main');
    hs=findall(0,'type','figure','tag','select_main');
    if ~isempty(ha)
      c=allchild(ha);
      hnumer=findobj(c,'flat','tag','sme_uic_onedit');
      hdenom=findobj(c,'flat','tag','sme_uic_odedit');
      fdtool('callback','sme','sme_cmd_autoorderpb','sme_uic_autoorderpb','auto');  
      fdtool('callback','sme','in_axes')
      hSt= findobj(c, 'tag', 'sme_uic_startpb');
      fdtool('callback','guidtawr','aided_main','DATA_INPUT','direct',fdata);
      fdtool('callback','guidtawr','aided_main','DONE_MODELS','direct',[]);
      if get(findobj(c, 'tag', 'sme_uic_showcb'),'value')
        %I don't find the cause, only visible window allows abort'
        set(findobj(c, 'tag', 'sme_mod_abortpb'),'enable','on')
      end
      tic, eval(get(hSt,'callback'))
      model=fdtool('callback','guidtard','aided_main','DONE_MODELS');
      if isempty(model)
        fdtool('callback','sme','status','Error: run aborted.')
        return
      end
    elseif ~isempty(hs)
      c=allchild(hs);
      hnumer=findobj(c,'flat','tag','sme_uic_onedit');
      hdenom=findobj(c,'flat','tag','sme_uic_odedit');
      fdtool('callback','sme','sme_cmd_autoorderpb','sme_uic_autoorderpb','auto');  
      %fdtool('callback','sme','in_axes')
      c=allchild(hs);
      hSt= findobj(c, 'tag', 'sme_uic_startpb');
      fdtool('callback','guidtawr','select_main','DATA_INPUT','direct',fdata);
      fdtool('callback','guidtawr','select_main','FINAL_DATA','direct',[]);
      if get(findobj(c, 'tag', 'sme_uic_showcb'),'value')
        %I don't find the cause, only visible window allows abort'
        set(findobj(c, 'tag', 'sme_mod_abortpb'),'enable','on')
      end
      tic, eval(get(hSt,'callback'))
      model=fdtool('callback','guidtard','select_main','FINAL_DATA');
      if isempty(model)
        fdtool('callback','sme','status','Error: run aborted.')
        return
      end
    else
      tic, model=autoorder(fdata,'s','',runmod); %toc;
    end
    if fdtool('callback','guiinfos','isdevelopment')&~isempty(model)
      hdemfig=findall(0,'type','figure','tag','AutoModelSelectionDemoFig');
      h=findall(hdemfig,'tag','popupmenu');
      demoN=strmatch(dem,get(h,'string'))-1;
      assignin('base','dno',size(get(h,'string'),1)-1);
      assignin('base','demoN',demoN); assignin('base','demostr',dem);
      global CORRTESTBASE
      if strcmp(CORRTESTBASE,'nlvar')
        if exist('autodemoresults2.mat'), evalin('base','load autodemoresults2.mat'), end
      else
        if exist('autodemoresults1.mat'), evalin('base','load autodemoresults1.mat'), end
      end
      evalin('base',...
        ['if ~exist(''autodemoresults''), autodemoresults=cell(dno,3); end, ',...
        '%autodemoresults(demoN:end,:)=[];'])
      assignin('base','models',model);
      evalin('base','autodemoresults{demoN,1}=demostr;')
      if iscell(model)
        evalin('base','autodemoresults{demoN,2}=models{1};')
        evalin('base','autodemoresults{demoN,3}=models{min(2,end)};')
      else
        evalin('base','autodemoresults{demoN,2}=models;')
        evalin('base','autodemoresults{demoN,3}=models;')
      end
      if strcmp(CORRTESTBASE,'nlvar')
        evalin('base','save autodemoresults2.mat autodemoresults')
      else
        evalin('base','save autodemoresults1.mat autodemoresults')
      end
    end
    while length(demonos)>length(truetimes)+1, truetimes=[truetimes;{NaN}]; end
    truetimes=[truetimes;{toc}];
    if strcmp(get(0,'diary'),'on')
      str='[demonos,timestrall,truetimes]';
      disp('List of demo numbers, expected run times, true run times:')
      table_of_run_times=[demonos,timestrall,truetimes]
      %save run times:
      %save testaod demonos timestrall truetimes rel_speed str
    end
    fdident('private','order_advice',model)
    if strncmp(version,'5',1)
      caller=getuprop(hFig,'CALLER_FCN');
    else
      caller=getappdata(hFig,'CALLER_FCN');
    end
    if strncmpi(caller,'sme',3)
      if ~isempty(findall(0,'type','figure','tag','select_main'))|...
           ~isempty(findall(0,'type','figure','tag','aided_main'))
        fdtool('callback','sme','restore_model',model,'autoorder')
      end
      %else
      %fdtool('callback','sme','sme_uic_donepb','sme_uic_donepb')
    end
    set(hA,'enable','off')
    figure(findall(0,'type','figure','tag', 'idbeard_tells_his_opinion'))
    
  elseif isequal(varargin{1}, 'help')
    hList=findobj(allchild(hFig), 'flat', 'Tag','ListBox');
    HelpStr={...
        'Demos for automatic model selection', ...
        '', ...
        '', ...
        '  In this demo you can find the result of several measurements and ', ...
        '  simulations. The data are all fiddata objects containing variance ', ...
        '  data calculated from multiple experiments.', ...
        '', ...
        '  Select one measurement or simulated data by pressing a pushbutton,', ...
        '  and read more about the specific experiment.', ...
        '', ...
        '  Press ''Start'' to invoke autoorder for the selected data.', ...
        '', ...
        '', ...
        'More topics are available, use the ''Topics'' pulldown menu.'};
    HelpStrCommLine={ ...
        'The automatic order/model selection algorithm can also be invoked from ',...
        'command line by typing:', ...
        '', ...
        '          model=autoorder(fdata,''s'') ', ...
        '', ...
        'The input argument ''fdata'' is an fdata object containing 1 (possibly)', ...
        '  averaged experiment with the appropriate variance and covariance data.', ...
        '  Note: ', ...
        '        This is the output of the Variance and/or Averaging block ', ...
        '        of the Frequency Domain Identification Toolbox GUI.', ...
        '', ...
        'The output is an fidmodel object containing usually two models.', ...
        '  The first one is a verified model, the second one is a lower order, ', ...
        '  somewhat poorer quality model. If the two models are identical, then', ...
        '  only one is returned.', ...
        '', ...
        '', ...
        'More topics are available, use the ''Topics'' pulldown menu.', ...
      };
    a=help('autoorder');
    helpwin({'autoorder', a;...
        'Command line option', HelpStrCommLine;...
        'Auto Model Selection Demo', char(HelpStr)});
    
  elseif isequal(varargin{1}, 'close')
    delete(hFig)
  end
end
%
%End of autoorder_demo
