function [props,values,indp] = idpvpstr(data,property)
%IDPVPSTR  Return the string of properties and their admissible values  
%       for the object class of data.
%
%       If property is given, the serial number of the property is returned
%
%       Usage: [props,values] = idpvpstr(data,property);
%       Copyright (c) I. Kollar and Vrije Universiteit Brussel, ELEC, 1997-2001
%       All rights reserved.
%       $Revision: $
%       Last modified: 19-Jun-2001
%All MODEL Properties
props = ['|Name|Version|Date|Notes|History|Data|Algorithm|UserData|',...
    'Variable|Representation|InterSample|Type|Coefficients|tauR|num|denom|C|D|F|',...
    'Ntr|Znum|Zdenom|Zntr|FreqVect|',...
    'Fs|Fscale|Delay|ioDelayMatrix|OutputDelay|InputDelay|',...
    'OutputChNumber|InputChNumber|OutputGroup|InputGroup|OutputName|InputName|',...
    'OutputUnit|InputUnit|',...
    'Covariance|FixedPar|ErrorWeighting|FitInfo|VarEt|',...
    'Consistency|Information|'];
%
%Now locate type-specific properties
inds=find(props=='|');
ind=findstr(props,['|Variable|']);
indvar=find(inds==ind);
%
inddel=[]; %indices of characters to be eliminated
indp=[]; %number of property to be eliminated
fdpnspec={'VarEt','C','D','F'};
for fdp=fdpnspec(:)'
  fdpi=fdp{1};
  ind=findstr(props,['|',fdpi,'|']);
  indp=[indp;find(inds==ind)];
  lfdpi=length(fdpi);
  inddel=[inddel;ind+1+[0:lfdpi]'];
end
%
%eliminate superfluous properties
if ~isempty(inddel), props(inddel)=''; end
%
if nargin>=2
  %if nargout>=2, error('Do not return values for single property'), end
  if ~isempty(data)
    if length(data)==1, data1=data; else data1=data(1); end
  else data1=[];
  end
  %
  if strcmpi(property,'B'), property='num';
  elseif strcmpi(property,'A'), property='den';
  end
  propf=findstr(lower(props),['|',lower(property)]);
  if length(propf)>1
    propffull=findstr(lower(props),['|',lower(property),'|']);
    if length(propffull)==1, propf=propffull; end
  end
  if length(propf)==1
    delims=find(props=='|'); idelim=find(delims==propf);
    propertyfull=props(delims(idelim)+1:delims(idelim+1)-1);
    values='';
    indp=find(find(props=='|')==propf);
  elseif length(propf)>1
    error(['Property ''',property,''' is ambiguous'])
  elseif length(propf)==0
    if isfield(struct(data1),property)
      propertyfull=property;
    else
      error(['Property ''',property,''' is not found for fidmodel object'])
    end
  else
    propertyfull=''; values=''; indp=[];
  end
  props=propertyfull;
  return
end
% Also return values if needed
if nargout>1,
  values = {'string'; ...
            'version number, set by the system';...
            'string (date + time)';...
	          'string';...
            'cell vector of strings';...
            'fitted data (fiddata object)';...
            'structure describing the algorithm';...
            'user-defined';...
            '[ ''z^-1'' | ''s'' | ''r'' | ''w'' ]';...
            '[ ''polynomial'' | ''orthopol'' ]';...
            '[ ''ZOH'' | ''band-limited'' | ''discrete'' ]';...
            'read-only';...
            '[ ''real'' | ''complex'' ]';...
            'tau-coefficient';...
            'cell array of numerators';...
            'cell array of denominators';...
            'cell array of noise numerators';...
            'cell array of noise denominators';...
            'cell array of left polynomials';...
            'cell array of transient numerator polynomials';...
            'array of weight vectors of numerator';...
            'array of weight vectors of denominator';...
            'array of weight vectors of transient numerator';...
            'column vector or array, freqs x channels';...
            'scalar, sampling frequency';...
            'scalar, scaling frequency';...
            'matrix of delay values';...
            'alternative name of delay';...
            'Ny-by-1 vector';...
            'Nu-by-1 vector';...
            'read-only: number of output channels';...
            'read-only: number of input channels';...
            'P-by-2 cell array if P output groups';...
            'M-by-2 cell array if M input groups';...
            'Ny-by-1 cell array of strings';...
            'Nu-by-1 cell array of strings';...
            'Ny-by-1 cell array of strings';...
            'Nu-by-1 cell array of strings';...
            'covariance array';...
            'nx2 array of fixed parameters';...
            '[''Linear'' , ''Nonlinear'' , ''Interpolated nonlinear'']';...
            'information on fit';...
            'array of estimated virtual noise variances'};
  if ~isempty(indp), values(indp)=[]; end
  %
  if (length(find(props=='|'))-1-2)~=size(values,1)
    error('inconsistent length')
  end
end
%end @fidmodel/idpvpstr.m
