function [props,values,indp] = idpropch(dat,property,mode)
%IDPROPCH  Return the string of properties and their admissible values  
%       for the object class of object DAT (iddat).
%       If property is not given, all properties are returned
%       MODE modifies the list of properties:
%       list mode: mode - 'set', 'all', ''
%       single property: '', 'noerror'

%       Copyright (c) I. Kollar and Vrije Universiteit Brussel, ELEC, 1998-2003
%       All rights reserved.
%       $Revision: $
%       Last modified: 23-Nov-2003

v=version; ip=findstr('.',v); if ip(1)==3, v=[setstr('a'-10+str2num(v(1:2))),v(3:end)]; end
if v(1)>='9', narginchk(1,3); %Matlab 2016a or later
else ni=nargin; error(nargchk(1,3,ni)), %earlier
end
if ~isa(dat,'iddat'), error('class is not ''iddat'''), end
if nargin<2, property=''; end
if nargin<3, mode=''; end
%
%All IDDAT Properties
props = ['|Name|Version|Date|Notes|History|Instrumentation|UserData|',...
    'Input|u|Output|y|InputName|OutputName|InputCharacter|OutputCharacter|',...
    'InputUnit|OutputUnit|InputScale|OutputScale|',...
    'PeriodLength|InputFrequencies|OutputFrequencies|Frequencies|',...
    'State|Synchronization|ChTypes|Characters|Data|Names|ExperimentName|',...
    'ReferenceData|Groups_Synchronized|Groups_Delayed|Groups_SamePower|Groups_FullMIMO|Groups|',...
    'Consistency|ExpNumber|ChNumber|InputChNumber|OutputChNumber|',...
    'Channels|Information|'];
%Property list also directly handled in iddat.m when defining fields

if nargout>1 %values also needed
  values = {...
      'string'; ...
      'version number, set by the system';...
      'string (date + time)';...
      'string';...
      'cell array of strings';...
      'structure';...
      'user-defined';...
      ...
      'array, size: inpchannels x samples, or cell array for more exp';...
      'same as Input';...
      'array, size: outpchannels x samples, or cell array for more exp';...
      'same as Output';...
      'cell array of strings';...
      'cell array of strings';...
      'cell array, [ ''ZOH'' | ''ZOHf'' | ''FOH'' | ''BL'' | ''Samples'' | ''Discrete'' ]';...
      'cell array, [ ''BL'' | ''Samples'' | ''Discrete'' ]';...
      ...
      'cell array of strings';...
      'cell array of strings';...
      'vector, length: channels';...
      'vector, length: channels';...
      ...
      'positive scalar, maybe NaN';...
      'vector or cell array';...
      'vector or cell array';...
      'vector or cell array';...
      ...
      'cell array, [ ''transient'' | ''steady-state'' ] ';...
      'string, [ ''on'' | ''delayed'' | ''samepower'' | ''mimo'' | '''' ] ';...
      'Hidden property';...
      'Hidden property';...
      'Hidden property';...
      'Hidden property';...
      'cell array';...
      ...
      'cell array of reference signals';...
      'cell array of arrays or cell arrays or groups';...
      'cell array of arrays or cell arrays or groups';...
      'cell array of arrays or cell arrays or groups';...
      'cell array of arrays or cell arrays or groups';...
      'Read or delete only';...
      ...
      'Read-only';...
      'Read-only';...
      'Read-only';...
      'Read-only';...
      'Read-only';...
      'Read-only';...
      'Read-only';...
      };
end
%
ind=find(props=='|'); pno=length(ind)-1;
if (nargout>1)&(length(values)~=pno)
  error('Different number of properties and values')
end
ip=findstr(props,'|PeriodLength|');
ic=findstr(props,'|Consistency|');
%
if isempty(mode) %props ready
elseif strcmp(mode,'all') %props ready
elseif strcmp(mode,'set') %only properties that can be set
  props=props(1:ic);
  if nargout>1
    di=sum(ind>ic);
    values=values(1:length(values)-di);
  end
end
if (nargin==1)|isempty(property), return, end
if strcmpi(property,'Information'), property='Channels'; end

if ~isempty(property) %Property to be determined
  if (length(property)>1)&any(findstr(lower(property(1)),'yu'))&~strncmpi(property,'use',3)
    if strcmp(lower(property(1)),'u'), property=['input',property(2:end)];
    elseif strcmp(lower(property(1)),'y'), property=['output',property(2:end)];
    end
  end
  if strncmpi(property,'inputda',7), property='input';
  elseif strncmpi(property,'outputda',8), property='output';
  end
  if strcmp(lower(property),'input')|strcmp(lower(property),'output')|...
      strcmp(lower(property),'name')|strcmp(lower(property),'groups')
    indpstr=findstr(lower(props),['|',lower(property),'|']);
  else  
    indpstr=findstr(lower(props),['|',lower(property)]);
  end
  if strcmp(lower(property),'u')|strcmp(lower(property),'y') %u or y
    if strcmp(lower(property),'u'), property='input';
    elseif strcmp(lower(property),'y'), property='output';
    end
    indpstr=findstr(lower(props),['|',lower(property),'|']);
  end
  if (length(indpstr)~=1)
    if ~strcmp(mode,'noerror')
      if isempty(indpstr)
        error(['Unidentifiable iddat property ',property])
      elseif length(indpstr)>1
        error(['Ambiguous iddat property ',property])
      end
    else
      props=[]; values=[]; indp=[]; return
    end
  end
  indi=min(find(ind>indpstr)); inde=ind(indi);
  props=props(indpstr+1:inde-1);
  if nargout>1
    no=length(find(ind<=indpstr)); values=values{no};
    indp=find(ind==indpstr);
  end
end

% end @iddat/idpropch.m
